/******************************************************************************
 *
 * File: FileprogLibDispatch.c
 *
 * Description:
 *  Main entry point and Dispatch table for the Fileprog Library. This
 *  module gets linked in with the Library source modules. It provides
 *  library dispatch table and the routine for initializing this table
 *  when the library is installed.
 *
 *  Don't change anything on that file. The library specifc code should be
 *  placed in FileLib.c
 *
 * Note:
 *  This file is based on the SampleLib example from Palm
 *****************************************************************************/

// Define this so we don't use the Palm OS pre-compiled headers.
// We can't use them here because we re-define some build options
#ifndef PILOT_PRECOMPILED_HEADERS_OFF
#define	PILOT_PRECOMPILED_HEADERS_OFF
#endif


// Define EMULATION_LEVEL.  __PALMOS_TRAPS__ was pre-defined by MW C Compiler
// Test it before we blow it away so we can properly define EMULATION_LEVEL.
#if __PALMOS_TRAPS__
		#define EMULATION_LEVEL		EMULATION_NONE		// building Pilot executable
#endif

// Now clear __PALMOS_TRAPS__ and define USE_TRAPS so headers do the right thing
// That is, we need to be able to get the addresses of FileprogLibrary routines.
// Other modules will access the routines through traps.
#undef __PALMOS_TRAPS__
#define __PALMOS_TRAPS__ 	0
#define	USE_TRAPS 	0


// Include Pilot headers
#include <PalmOS.h>


// Our library public definitions (library API)
#include "FileLib.h"


// Private routines

#if EMULATION_LEVEL == EMULATION_NONE
	#define FileprogLibInstall		__Startup__
#endif


// Local prototypes

Err FileprogLibInstall(UInt16 refNum, SysLibTblEntryPtr entryP);
static MemPtr	asm FileprogLibDispatchTable(void);



/***********************************************************************
 * FUNCTION:
 *
 * DESCRIPTION:
 *
 * PARAMETERS:   
 *
 * RETURNS:     
 ***********************************************************************/
Err FileprogLibInstall(UInt16 refNum, SysLibTblEntryPtr entryP)
{
	// Install pointer to our dispatch table
	entryP->dispatchTblP = (MemPtr*)FileprogLibDispatchTable();
	
	// Initialize globals pointer to zero (we will set up our library
	// globals in the library "open" call).
	entryP->globalsP = 0;

	return 0;
	
}


/***********************************************************************
 * FUNCTION:
 *
 * DESCRIPTION:
 *
 * PARAMETERS:   
 *
 * RETURNS:     
 ***********************************************************************/
// First, define the size of the jump instruction
#if EMULATION_LEVEL == EMULATION_NONE
	#define prvJmpSize		4					// Palm OS presently uses short jumps
#elif EMULATION_LEVEL == EMULATION_MAC
	#define prvJmpSize		6					// Our Mac builds use long jumps
#else
	#error unsupported emulation mode
#endif	// EMULATION_LEVEL...

// Now, define a macro for offset list entries
#define libDispatchEntry(index)		(kOffset+((index)*prvJmpSize))


// Finally, define the size of the dispatch table's offset list --
// it is equal to the size of each entry (which is presently 2 bytes) times
// the number of entries in the offset list (***including the @Name entry***).


// 5 = 1+4 for the mandatory functions in the library interface (4) and
// 1 is due to
// This is based on fileprogLibTrapLast being the last trap in the 
// trap enumeration. Don't change that.
#define	kOffset (2*(fileprogLibTrapLast-sysLibTrapCustom+5))

static MemPtr	asm FileprogLibDispatchTable(void)
{
	LEA		@Table, A0								// table ptr
	RTS												// exit with it

@Table:
	// Offset to library name
	DC.W		@Name
	
	//
	// Library function dispatch entries
	//
	// ***IMPORTANT***
	// The index parameters passed to the macro libDispatchEntry
	// must be numbered consecutively, beginning with zero.
	//
	// The hard-wired values need to be used for offsets because the MW SDK tools do
	// not support label subtraction.
	//
	
	// Standard traps
	DC.W		libDispatchEntry(0)					// Open
	DC.W		libDispatchEntry(1)					// Close
	DC.W		libDispatchEntry(2)					// Sleep
	DC.W		libDispatchEntry(3)					// Wake
	
	// Start of the Custom traps
	DC.W		libDispatchEntry(4)					// FileprogLibGetLibAPIVersion
	DC.W		libDispatchEntry(5)					// FileprogLibGetLibDescription
	DC.W		libDispatchEntry(6)					// FileprogLibRunDbRam
	DC.W		libDispatchEntry(7)					// FileprogLibRunDbCard
	DC.W		libDispatchEntry(8)					// FileprogLibProlog
	DC.W		libDispatchEntry(9)					// FileprogLibEpilog

// Standard library function handlers
@GotoOpen:
	JMP 		FileprogLibOpen
@GotoClose:
	JMP 		FileprogLibClose
@GotoSleep:
	JMP 		FileprogLibSleep
@GotoWake:
	JMP 		FileprogLibWake
	
// Customer library function handlers
@GotoGetLibAPIVersion:
	JMP 		FileprogLibGetLibAPIVersion
@GotoGetLibDescription:
	JMP 		FileprogLibGetLibDescription
@GotoRunDbRam:
	JMP 		FileprogLibRunDbRam
@GotoRunDbCard:
	JMP		    FileprogLibRunDbCard
@GotoProlog:
	JMP		    FileprogLibProlog
@GotoEpilog:
	JMP		    FileprogLibEpilog

	
@Name:
	DC.B		fileprogLibName							// This name identifies the library.  Apps
														// can pass it to SysLibFind() to check if the
														// library is already installed
	
}
